@@ -25,7 +25,7 @@
 #  include "config.h"
 #endif
 
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 #include "gstsfsrc.h"
 
@@ -83,8 +83,7 @@ gst_sf_src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sf_src_factory));
+  gst_element_class_add_static_pad_template (gstelement_class, &sf_src_factory);
 
   gst_element_class_set_static_metadata (gstelement_class, "Sndfile source",
       "Source/Audio",
@@ -107,7 +106,8 @@ gst_sf_src_class_init (GstSFSrcClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_LOCATION,
       g_param_spec_string ("location", "File Location",
-          "Location of the file to read", NULL, G_PARAM_READWRITE));
+          "Location of the file to read", NULL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_sf_src_finalize);
 
@@ -146,8 +146,7 @@ gst_sf_src_set_location (GstSFSrc * this, const gchar * location)
   if (this->file)
     goto was_open;
 
-  if (this->location)
-    g_free (this->location);
+  g_free (this->location);
 
   this->location = location ? g_strdup (location) : NULL;
 
