@@ -1,5 +1,5 @@
 /* GStreamer Adaptive Multi-Rate Wide-Band (AMR-WB) plugin
- * Copyright (C) 2006 Edgard Lima <edgard.lima@indt.org.br>
+ * Copyright (C) 2006 Edgard Lima <edgard.lima@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,19 +19,19 @@
 
 /**
  * SECTION:element-voamrwbenc
+ * @title: voamrwbenc
  * @see_also: #GstAmrWbDec, #GstAmrWbParse
  *
- * AMR wideband encoder based on the 
- * <ulink url="http://www.penguin.cz/~utx/amr">reference codec implementation</ulink>.
- * 
- * <refsect2>
- * <title>Example launch line</title>
+ * AMR wideband encoder based on the
+ * [reference codec implementation](http://www.penguin.cz/~utx/amr).
+ *
+ * ## Example launch line
  * |[
  * gst-launch filesrc location=abc.wav ! wavparse ! audioresample ! audioconvert ! voamrwbenc ! filesink location=abc.amr
  * ]|
  * Please note that the above stream misses the header, that is needed to play
  * the stream.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,16 +40,18 @@
 
 #include "gstvoamrwbenc.h"
 
+#include <string.h>
+
 #define MR660  0
 #define MR885  1
 #define MR1265 2
-#define MR1425 2
-#define MR1585 3
-#define MR1825 4
-#define MR1985 5
-#define MR2305 6
-#define MR2385 7
-#define MRDTX  8
+#define MR1425 3
+#define MR1585 4
+#define MR1825 5
+#define MR1985 6
+#define MR2305 7
+#define MR2385 8
+#define MRDTX  9
 
 #define L_FRAME16k      320     /* Frame size at 16kHz  */
 
@@ -115,6 +117,8 @@ static GstFlowReturn gst_voamrwbenc_handle_frame (GstAudioEncoder * enc,
     GstBuffer * in_buf);
 
 G_DEFINE_TYPE (GstVoAmrWbEnc, gst_voamrwbenc, GST_TYPE_AUDIO_ENCODER);
+GST_ELEMENT_REGISTER_DEFINE (voamrwbenc, "voamrwbenc",
+    GST_RANK_SECONDARY, GST_TYPE_VOAMRWBENC);
 
 static void
 gst_voamrwbenc_set_property (GObject * object, guint prop_id,
@@ -183,6 +187,8 @@ gst_voamrwbenc_class_init (GstVoAmrWbEncClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (gst_voamrwbenc_debug, "voamrwbenc", 0,
       "voamrwb encoder");
+
+  gst_type_mark_as_plugin_api (GST_VOAMRWBENC_BANDMODE_TYPE, 0);
 }
 
 static void
@@ -285,18 +291,22 @@ gst_voamrwbenc_handle_frame (GstAudioEncoder * benc, GstBuffer * buffer)
 
   gst_buffer_map (buffer, &map, GST_MAP_READ);
 
-  if (G_UNLIKELY (map.size < buffer_size)) {
-    GST_DEBUG_OBJECT (amrwbenc, "discarding trailing data %d", (gint) map.size);
-    gst_buffer_unmap (buffer, &map);
-    ret = gst_audio_encoder_finish_frame (benc, NULL, -1);
-    goto done;
-  }
-
   out = gst_buffer_new_and_alloc (buffer_size);
   gst_buffer_map (out, &omap, GST_MAP_WRITE);
+
   /* encode */
-  outsize = E_IF_encode (amrwbenc->handle, amrwbenc->bandmode,
-      (const short *) map.data, (unsigned char *) omap.data, 0);
+  if (G_UNLIKELY (map.size < buffer_size)) {
+    short input_buffer[L_FRAME16k] = { 0, };
+
+    GST_DEBUG_OBJECT (amrwbenc, "add silence to packet of size %d",
+        (gint) map.size);
+    memcpy ((void *) input_buffer, map.data, map.size);
+    outsize = E_IF_encode (amrwbenc->handle, amrwbenc->bandmode,
+        (const short *) input_buffer, (unsigned char *) omap.data, 0);
+  } else {
+    outsize = E_IF_encode (amrwbenc->handle, amrwbenc->bandmode,
+        (const short *) map.data, (unsigned char *) omap.data, 0);
+  }
 
   GST_LOG_OBJECT (amrwbenc, "encoded to %d bytes", outsize);
   gst_buffer_unmap (out, &omap);
