@@ -70,10 +70,19 @@ struct _GstVulkanPhysicalDevicePrivate
 #if defined (VK_API_VERSION_1_3)
   VkPhysicalDeviceVulkan13Features features13;
   VkPhysicalDeviceVulkan13Properties properties13;
+#endif
+#if defined (VK_KHR_sampler_ycbcr_conversion)
+  VkPhysicalDeviceSamplerYcbcrConversionFeaturesKHR sampler_ycbcr_conversion;
+#endif
+#if defined (VK_KHR_synchronization2)
+  VkPhysicalDeviceSynchronization2FeaturesKHR synchronization2;
+#endif
+#if defined (VK_KHR_timeline_semaphore)
+  VkPhysicalDeviceTimelineSemaphoreFeaturesKHR timeline_semaphore;
+#endif
 #if defined (VK_KHR_video_maintenance1)
   VkPhysicalDeviceVideoMaintenance1FeaturesKHR videomaintenance1;
 #endif
-#endif
 };
 
 static void
@@ -129,6 +138,7 @@ gst_vulkan_physical_device_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_INSTANCE:
+      /* G_PARAM_CONSTRUCT_ONLY */
       device->instance = g_value_dup_object (value);
       break;
     case PROP_DEVICE_ID:{
@@ -203,11 +213,26 @@ gst_vulkan_physical_device_init (GstVulkanPhysicalDevice * device)
   priv->features13.sType =
       VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES;
   priv->features12.pNext = &priv->features13;
+#endif
+#if defined (VK_KHR_sampler_ycbcr_conversion)
+  priv->sampler_ycbcr_conversion.sType =
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_YCBCR_CONVERSION_FEATURES_KHR;
+  vk_link_struct (&priv->features12, &priv->sampler_ycbcr_conversion);
+#endif
+#if defined (VK_KHR_synchronization2)
+  priv->synchronization2.sType =
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SYNCHRONIZATION_2_FEATURES_KHR;
+  vk_link_struct (&priv->features12, &priv->synchronization2);
+#endif
+#if defined (VK_KHR_timeline_semaphore)
+  priv->timeline_semaphore.sType =
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_FEATURES_KHR;
+  vk_link_struct (&priv->features12, &priv->timeline_semaphore);
+#endif
 #if defined (VK_KHR_video_maintenance1)
   priv->videomaintenance1.sType =
       VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_MAINTENANCE_1_FEATURES_KHR;
-  priv->features13.pNext = &priv->videomaintenance1;
-#endif
+  vk_link_struct (&priv->features12, &priv->videomaintenance1);
 #endif
 }
 
@@ -486,17 +511,43 @@ dump_features13 (GstVulkanPhysicalDevice * device,
   DEBUG_BOOL_STRUCT ("support for (1.3)", features, maintenance4);
   /* *INDENT-ON* */
 }
-
-#if defined(VK_KHR_video_maintenance1)
-static void
-dump_videomaintenance1 (GstVulkanPhysicalDevice * device,
-    VkPhysicalDeviceVideoMaintenance1FeaturesKHR * features)
-{
-  DEBUG_BOOL_STRUCT ("support for (1.3)", features, videoMaintenance1);
-}
-#endif
 #endif /* defined (VK_API_VERSION_1_3) */
 
+static void
+dump_extras (GstVulkanPhysicalDevice * device, VkBaseOutStructure * chain)
+{
+  GstVulkanPhysicalDevicePrivate *priv = GET_PRIV (device);
+
+#if defined (VK_KHR_sampler_ycbcr_conversion)
+  if (chain->sType ==
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_YCBCR_CONVERSION_FEATURES_KHR) {
+    DEBUG_BOOL_STRUCT ("support for", &priv->sampler_ycbcr_conversion,
+        samplerYcbcrConversion);
+  }
+#endif
+#if defined (VK_KHR_synchronization2)
+  if (chain->sType ==
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SYNCHRONIZATION_2_FEATURES_KHR) {
+    DEBUG_BOOL_STRUCT ("support for", &priv->synchronization2,
+        synchronization2);
+  }
+#endif
+#if defined (VK_KHR_timeline_semaphore)
+  if (chain->sType ==
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_FEATURES_KHR) {
+    DEBUG_BOOL_STRUCT ("support for", &priv->timeline_semaphore,
+        timelineSemaphore);
+  }
+#endif
+#if defined (VK_KHR_video_maintenance1)
+  if (chain->sType ==
+      VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_MAINTENANCE_1_FEATURES_KHR) {
+    DEBUG_BOOL_STRUCT ("support for", &priv->videomaintenance1,
+        videoMaintenance1);
+  }
+#endif
+}
+
 static gboolean
 dump_features (GstVulkanPhysicalDevice * device, GError ** error)
 {
@@ -504,7 +555,7 @@ dump_features (GstVulkanPhysicalDevice * device, GError ** error)
   GstVulkanPhysicalDevicePrivate *priv = GET_PRIV (device);
   VkBaseOutStructure *iter;
 
-  if (gst_vulkan_instance_check_version (device->instance, 1, 2, 0)) {
+  if (gst_vulkan_physical_device_check_api_version (device, 1, 2, 0)) {
     for (iter = (VkBaseOutStructure *) & priv->features10; iter;
         iter = iter->pNext) {
       if (iter->sType == VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2)
@@ -517,18 +568,13 @@ dump_features (GstVulkanPhysicalDevice * device, GError ** error)
           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES)
         dump_features12 (device, (VkPhysicalDeviceVulkan12Features *) iter);
 #if defined (VK_API_VERSION_1_3)
-      else if (gst_vulkan_instance_check_version (device->instance, 1, 3, 0)
+      else if (gst_vulkan_physical_device_check_api_version (device, 1, 3, 0)
           && iter->sType ==
           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES)
         dump_features13 (device, (VkPhysicalDeviceVulkan13Features *) iter);
-#if defined(VK_KHR_video_maintenance1)
-      else if (gst_vulkan_instance_check_version (device->instance, 1, 3, 283)
-          && iter->sType ==
-          VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_MAINTENANCE_1_FEATURES_KHR)
-        dump_videomaintenance1 (device,
-            (VkPhysicalDeviceVideoMaintenance1FeaturesKHR *) iter);
-#endif
 #endif
+      else
+        dump_extras (device, iter);
     }
   } else
 #endif
@@ -909,7 +955,7 @@ physical_device_info (GstVulkanPhysicalDevice * device, GError ** error)
     return FALSE;
 
 #if defined (VK_API_VERSION_1_2)
-  if (gst_vulkan_instance_check_version (device->instance, 1, 2, 0)) {
+  if (gst_vulkan_physical_device_check_api_version (device, 1, 2, 0)) {
     for (iter = (VkBaseOutStructure *) & priv->properties10; iter;
         iter = iter->pNext) {
       if (iter->sType ==
@@ -919,7 +965,7 @@ physical_device_info (GstVulkanPhysicalDevice * device, GError ** error)
           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_PROPERTIES)
         dump_properties12 (device, (VkPhysicalDeviceVulkan12Properties *) iter);
 #if defined (VK_API_VERSION_1_3)
-      else if (gst_vulkan_instance_check_version (device->instance, 1, 3, 0)
+      else if (gst_vulkan_physical_device_check_api_version (device, 1, 3, 0)
           && iter->sType ==
           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_PROPERTIES)
         dump_properties13 (device, (VkPhysicalDeviceVulkan13Properties *) iter);
@@ -992,7 +1038,7 @@ gst_vulkan_physical_device_fill_info (GstVulkanPhysicalDevice * device,
 
   vkGetPhysicalDeviceProperties (device->device, &device->properties);
 #if defined (VK_API_VERSION_1_2)
-  if (gst_vulkan_instance_check_version (device->instance, 1, 2, 0)) {
+  if (gst_vulkan_physical_device_check_api_version (device, 1, 2, 0)) {
     PFN_vkGetPhysicalDeviceProperties2 get_props2;
     PFN_vkGetPhysicalDeviceMemoryProperties2 get_mem_props2;
     PFN_vkGetPhysicalDeviceFeatures2 get_features2;
@@ -1262,8 +1308,58 @@ gst_vulkan_physical_device_get_features (GstVulkanPhysicalDevice * device)
   g_return_val_if_fail (GST_IS_VULKAN_PHYSICAL_DEVICE (device), FALSE);
 
   priv = GET_PRIV (device);
-  if (gst_vulkan_instance_check_version (device->instance, 1, 2, 0))
+  if (gst_vulkan_physical_device_check_api_version (device, 1, 2, 0))
     return &priv->features10;
 #endif
   return NULL;
 }
+
+/**
+ * gst_vulkan_physical_device_get_api_version:
+ * @device: a #GstVulkanPhysicalDevice
+ * @major: (out): major version
+ * @minor: (out): minor version
+ * @patch: (out): patch version
+ *
+ * Retrieves the advertised Vulkan API version of the #GstVulkanPhysicalDevice.
+ *
+ * Since: 1.26
+ */
+void
+gst_vulkan_physical_device_get_api_version (GstVulkanPhysicalDevice * device,
+    guint * major, guint * minor, guint * patch)
+{
+  if (major)
+    *major = VK_VERSION_MAJOR (device->properties.apiVersion);
+  if (minor)
+    *minor = VK_VERSION_MINOR (device->properties.apiVersion);
+  if (patch)
+    *patch = VK_VERSION_PATCH (device->properties.apiVersion);
+}
+
+/**
+ * gst_vulkan_physical_device_check_api_version:
+ * @device: a #GstVulkanPhysicalDevice
+ * @major: the API major version to check
+ * @minor: the API minor version to check
+ * @patch: the API patch version to check
+ *
+ * Note: This is the intersection of the exposed supported API version as would
+ * be returned by gst_vulkan_physical_device_get_api_version() and
+ * gst_vulkan_instance_check_version().  The latter will take into account any
+ * requested API version and may result in a different result than directly
+ * comparing against gst_vulkan_instance_get_version().
+ *
+ * Returns: whether the #GstVulkanPhysicalDevice supports the version specified
+ *          by @major, @minor and @patch.
+ *
+ * Since: 1.26
+ */
+gboolean
+gst_vulkan_physical_device_check_api_version (GstVulkanPhysicalDevice * device,
+    guint major, guint minor, guint patch)
+{
+  return VK_MAKE_VERSION (major, minor, patch) <= device->properties.apiVersion
+      && gst_vulkan_instance_check_api_version (device->instance, major, minor,
+      patch);
+}
