@@ -20,17 +20,17 @@
  */
 /**
  * SECTION:element-synaescope
+ * @title: synaescope
  * @see_also: goom
  *
  * Synaescope is an audio visualisation element. It analyzes frequencies and
  * out-of phase properties of audio and draws this as clouds of stars.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 audiotestsrc ! audioconvert ! synaescope ! ximagesink
  * ]|
- * </refsect2>
+ *
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,15 +38,17 @@
 
 #include "gstsynaescope.h"
 
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+#define RGB_ORDER "xRGB"
+#else
+#define RGB_ORDER "BGRx"
+#endif
+
 static GstStaticPadTemplate gst_synae_scope_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("xRGB"))
-#else
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("BGRx"))
-#endif
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (RGB_ORDER))
     );
 
 static GstStaticPadTemplate gst_synae_scope_sink_template =
@@ -71,7 +73,11 @@ static gboolean gst_synae_scope_render (GstAudioVisualizer * scope,
     GstBuffer * audio, GstVideoFrame * video);
 
 
-G_DEFINE_TYPE (GstSynaeScope, gst_synae_scope, GST_TYPE_AUDIO_VISUALIZER);
+G_DEFINE_TYPE_WITH_CODE (GstSynaeScope, gst_synae_scope,
+    GST_TYPE_AUDIO_VISUALIZER, GST_DEBUG_CATEGORY_INIT (synae_scope_debug,
+        "synaescope", 0, "synaescope"););
+GST_ELEMENT_REGISTER_DEFINE (synaescope, "synaescope", GST_RANK_NONE,
+    GST_TYPE_SYNAE_SCOPE);
 
 static void
 gst_synae_scope_class_init (GstSynaeScopeClass * g_class)
@@ -87,10 +93,10 @@ gst_synae_scope_class_init (GstSynaeScopeClass * g_class)
       "Creates video visualizations of audio input, using stereo and pitch information",
       "Stefan Kost <ensonic@users.sf.net>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_synae_scope_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_synae_scope_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_synae_scope_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_synae_scope_sink_template);
 
   scope_class->setup = GST_DEBUG_FUNCPTR (gst_synae_scope_setup);
   scope_class->render = GST_DEBUG_FUNCPTR (gst_synae_scope_render);
@@ -266,7 +272,7 @@ gst_synae_scope_render (GstAudioVisualizer * bscope, GstBuffer * audio,
     fc = r + l;
 
     x = (guint) (r * w / fc);
-    /* the brighness scaling factor was picked by experimenting */
+    /* the brightness scaling factor was picked by experimenting */
     br = b * fc * 0.01;
 
     br1 = br * (clarity + 128) >> 8;
@@ -307,12 +313,3 @@ gst_synae_scope_render (GstAudioVisualizer * bscope, GstBuffer * audio,
 
   return TRUE;
 }
-
-gboolean
-gst_synae_scope_plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (synae_scope_debug, "synaescope", 0, "synaescope");
-
-  return gst_element_register (plugin, "synaescope", GST_RANK_NONE,
-      GST_TYPE_SYNAE_SCOPE);
-}
