@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,31 +26,16 @@
 #include "gstcoloreffects.h"
 #include "gstchromahold.h"
 
-struct _elements_entry
-{
-  const gchar *name;
-    GType (*type) (void);
-};
-
-static const struct _elements_entry _elements[] = {
-  {"coloreffects", gst_color_effects_get_type},
-  {"chromahold", gst_chroma_hold_get_type},
-  {NULL, 0},
-};
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  gint i = 0;
+  gboolean ret = FALSE;
 
-  while (_elements[i].name) {
-    if (!gst_element_register (plugin, _elements[i].name,
-            GST_RANK_NONE, (_elements[i].type) ()))
-      return FALSE;
-    i++;
-  }
+  ret |= GST_ELEMENT_REGISTER (coloreffects, plugin);
+  ret |= GST_ELEMENT_REGISTER (chromahold, plugin);
 
-  return TRUE;
+  return ret;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
