@@ -43,16 +43,16 @@
 
 /**
  * SECTION:element-stretch
+ * @title: stretch
  * @see_also: geometrictransform
  *
  * The stretch element stretches the image in a circle around the center point.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v videotestsrc ! stretch ! videoconvert ! autovideosink
  * ]|
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -79,6 +79,9 @@ enum
 
 #define gst_stretch_parent_class parent_class
 G_DEFINE_TYPE (GstStretch, gst_stretch, GST_TYPE_CIRCLE_GEOMETRIC_TRANSFORM);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (stretch, "stretch", GST_RANK_NONE,
+    GST_TYPE_STRETCH, GST_DEBUG_CATEGORY_INIT (gst_stretch_debug, "stretch", 0,
+        "stretch"));
 
 static void
 gst_stretch_set_property (GObject * object, guint prop_id, const GValue * value,
@@ -139,7 +142,7 @@ stretch_map (GstGeometricTransform * gt, gint x, gint y, gdouble * in_x,
   gdouble height = gt->height;
   gdouble a, b;
 
-  /* normalize in ((-1.0, -1.0), (1.0, 1.0) and traslate the center */
+  /* normalize in ((-1.0, -1.0), (1.0, 1.0) and translate the center */
   norm_x = 2.0 * (x / width - cgt->x_center);
   norm_y = 2.0 * (y / height - cgt->y_center);
 
@@ -209,12 +212,3 @@ gst_stretch_init (GstStretch * filter)
   filter->intensity = DEFAULT_INTENSITY;
   gt->off_edge_pixels = GST_GT_OFF_EDGES_PIXELS_CLAMP;
 }
-
-gboolean
-gst_stretch_plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (gst_stretch_debug, "stretch", 0, "stretch");
-
-  return gst_element_register (plugin, "stretch", GST_RANK_NONE,
-      GST_TYPE_STRETCH);
-}
