@@ -684,6 +684,7 @@ mpegts_base_program_remove_stream (MpegTSBase * base,
   program->streams[pid] = NULL;
 }
 
+#if 0                           /* Smart-update disabled */
 /* Check if pmtstream is already present in the program */
 static inline gboolean
 _stream_in_pmt (const GstMpegtsPMT * pmt, MpegTSBaseStream * stream)
@@ -768,7 +769,7 @@ mpegts_base_update_program (MpegTSBase * base, MpegTSBaseProgram * program,
   g_list_free (toremove);
   return TRUE;
 }
-
+#endif
 
 static gboolean
 _stream_is_private_section (const GstMpegtsPMT * pmt,
@@ -784,6 +785,7 @@ _stream_is_private_section (const GstMpegtsPMT * pmt,
       if (registration_id != DRF_ID_CUEI && registration_id != DRF_ID_ETV1)
         return FALSE;
     }
+      /* FALLTHROUGH */
     case GST_MPEGTS_STREAM_TYPE_PRIVATE_SECTIONS:
     case GST_MPEGTS_STREAM_TYPE_MHEG:
     case GST_MPEGTS_STREAM_TYPE_DSM_CC:
@@ -814,7 +816,6 @@ mpegts_base_is_same_program (MpegTSBase * base, MpegTSBaseProgram * oldprogram,
 {
   guint i, nbstreams;
   MpegTSBaseStream *oldstream;
-  gboolean sawpcrpid = FALSE;
 
   if (oldprogram->pmt_pid != new_pmt_pid) {
     GST_DEBUG ("Different pmt_pid (new:0x%04x, old:0x%04x)", new_pmt_pid,
@@ -828,7 +829,6 @@ mpegts_base_is_same_program (MpegTSBase * base, MpegTSBaseProgram * oldprogram,
     return FALSE;
   }
 
-  /* Check the streams */
   nbstreams = new_pmt->streams->len;
   for (i = 0; i < nbstreams; ++i) {
     GstMpegtsPMTStream *stream = g_ptr_array_index (new_pmt->streams, i);
@@ -844,17 +844,14 @@ mpegts_base_is_same_program (MpegTSBase * base, MpegTSBaseProgram * oldprogram,
           stream->pid, stream->stream_type, oldstream->stream_type);
       return FALSE;
     }
-    if (stream->pid == oldprogram->pcr_pid)
-      sawpcrpid = TRUE;
   }
 
-  /* If we have a PCR PID and the pcr is not shared with an existing stream, we'll have one extra stream */
-  if (!sawpcrpid && !base->ignore_pcr)
-    nbstreams += 1;
+  /* We can now just check the number of streams from each PMT. The check for
+   * PCR was already done previously */
 
-  if (nbstreams != g_list_length (oldprogram->stream_list)) {
-    GST_DEBUG ("Different number of streams (new:%d, old:%d)",
-        nbstreams, g_list_length (oldprogram->stream_list));
+  if (nbstreams != oldprogram->pmt->streams->len) {
+    GST_DEBUG ("Different number of streams (new:%d, old:%u)",
+        nbstreams, oldprogram->pmt->streams->len);
     return FALSE;
   }
 
@@ -862,6 +859,7 @@ mpegts_base_is_same_program (MpegTSBase * base, MpegTSBaseProgram * oldprogram,
   return TRUE;
 }
 
+#if 0                           /* Smart-update disabled */
 /* Return TRUE if program is an update
  *
  * A program is equal if:
@@ -923,6 +921,7 @@ mpegts_base_is_program_update (MpegTSBase * base,
   GST_DEBUG ("Program is not an update of the previous one");
   return FALSE;
 }
+#endif
 
 static void
 mpegts_base_deactivate_program (MpegTSBase * base, MpegTSBaseProgram * program)
@@ -1007,9 +1006,7 @@ mpegts_base_activate_program (MpegTSBase * base, MpegTSBaseProgram * program,
         MPEGTS_BIT_SET (base->known_psi, stream->pid);
     } else {
       if (G_UNLIKELY (MPEGTS_BIT_IS_SET (base->is_pes, stream->pid)))
-        GST_FIXME
-            ("Refcounting issue. Setting twice a PID (0x%04x) as known PES",
-            stream->pid);
+        GST_DEBUG ("Setting twice a PID (0x%04x) as known PES", stream->pid);
       if (G_UNLIKELY (MPEGTS_BIT_IS_SET (base->known_psi, stream->pid))) {
         GST_FIXME
             ("Refcounting issue. Setting a known PSI PID (0x%04x) as known PES",
@@ -1208,6 +1205,8 @@ mpegts_base_apply_pmt (MpegTSBase * base, GstMpegtsSection * section)
               pmt)))
     goto same_program;
 
+#if 0
+  /* parsebin doesn't support program update properly. Disable this feature for now */
   if (base->streams_aware
       && mpegts_base_is_program_update (base, old_program, section->pid, pmt)) {
     GST_FIXME ("We are streams_aware and new program is an update");
@@ -1215,6 +1214,7 @@ mpegts_base_apply_pmt (MpegTSBase * base, GstMpegtsSection * section)
     mpegts_base_update_program (base, old_program, section, pmt);
     goto beach;
   }
+#endif
 
   /* If the current program is active, this means we have a new program */
   if (old_program->active) {
@@ -1247,7 +1247,9 @@ mpegts_base_apply_pmt (MpegTSBase * base, GstMpegtsSection * section)
   mpegts_base_activate_program (base, program, section->pid, section, pmt,
       initial_program);
 
+#if 0                           /* Smart-update disabled */
 beach:
+#endif
   GST_DEBUG ("Done activating program");
   return TRUE;
 
