@@ -30,14 +30,12 @@
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "rtponviftimestamp", GST_RANK_NONE,
-          GST_TYPE_RTP_ONVIF_TIMESTAMP))
-    return FALSE;
-  if (!gst_element_register (plugin, "rtponvifparse", GST_RANK_NONE,
-          GST_TYPE_RTP_ONVIF_PARSE))
-    return FALSE;
+  gboolean ret = FALSE;
 
-  return TRUE;
+  ret |= GST_ELEMENT_REGISTER (rtponviftimestamp, plugin);
+  ret |= GST_ELEMENT_REGISTER (rtponvifparse, plugin);
+
+  return ret;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
