@@ -39,7 +39,7 @@ enum
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("text/vtt"));
+    GST_STATIC_CAPS ("application/x-subtitle-vtt"));
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -58,6 +58,8 @@ static void gst_webvtt_enc_set_property (GObject * object, guint prop_id,
 
 #define parent_class gst_webvtt_enc_parent_class
 G_DEFINE_TYPE (GstWebvttEnc, gst_webvtt_enc, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE (webvttenc, "webvttenc", GST_RANK_NONE,
+    GST_TYPE_WEBVTT_ENC);
 
 static void
 gst_webvtt_enc_append_timestamp_to_string (GstClockTime timestamp,
@@ -266,10 +268,8 @@ gst_webvtt_enc_class_init (GstWebvttEncClass * klass)
           0,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (element_class, &sink_template);
+  gst_element_class_add_static_pad_template (element_class, &src_template);
 
   gst_element_class_set_static_metadata (element_class,
       "WebVTT encoder", "Codec/Encoder/Subtitle",
