@@ -228,6 +228,7 @@ gst_kms_allocator_constructed (GObject * obj)
   GstKMSAllocator *alloc;
 
   alloc = GST_KMS_ALLOCATOR (obj);
+  G_OBJECT_CLASS (parent_class)->constructed (obj);
 
   /* Should be called after the properties are set */
   g_assert (check_fd (alloc));
@@ -274,7 +275,7 @@ gst_kms_allocator_class_init (GstKMSAllocatorClass * klass)
 
   g_props[PROP_DRM_FD] = g_param_spec_int ("drm-fd", "DRM fd",
       "DRM file descriptor", -1, G_MAXINT, -1,
-      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class, PROP_N, g_props);
 }
