@@ -53,28 +53,33 @@
 #include <mfxjpeg.h>
 
 #include "gstmsdkmjpegdec.h"
-#include "gstmsdkvideomemory.h"
 
 #include <gst/pbutils/pbutils.h>
 
 GST_DEBUG_CATEGORY_EXTERN (gst_msdkmjpegdec_debug);
 #define GST_CAT_DEFAULT gst_msdkmjpegdec_debug
 
-static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("image/jpeg, "
-        "width = (int) [ 1, MAX ], height = (int) [ 1, MAX ], parsed = true ")
-    );
+#define GST_MSDKMJPEGDEC(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_FROM_INSTANCE (obj), GstMsdkMJPEGDec))
+#define GST_MSDKMJPEGDEC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_FROM_CLASS (klass), GstMsdkMJPEGDecClass))
+#define GST_IS_MSDKMJPEGDEC(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_FROM_INSTANCE (obj)))
+#define GST_IS_MSDKMJPEGDEC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_FROM_CLASS (klass)))
 
-static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_MSDK_CAPS_STR ("{ NV12, YUY2 }", "{ NV12, YUY2 }"))
-    );
+/* *INDENT-OFF* */
+static const gchar *doc_src_caps_str =
+    GST_VIDEO_CAPS_MAKE ("{ NV12, YUY2, BGRA }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:DMABuf",
+        "{ NV12, YUY2, BGRA }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:VAMemory", "{ NV12 }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:D3D11Memory", "{ NV12 }");
+/* *INDENT-ON* */
 
-#define gst_msdkmjpegdec_parent_class parent_class
-G_DEFINE_TYPE (GstMsdkMJPEGDec, gst_msdkmjpegdec, GST_TYPE_MSDKDEC);
+static const gchar *doc_sink_caps_str = "image/jpeg";
+
+static GstElementClass *parent_class = NULL;
 
 static gboolean
 gst_msdkmjpegdec_configure (GstMsdkDec * decoder)
@@ -106,6 +111,7 @@ static gboolean
 gst_msdkmjpegdec_post_configure (GstMsdkDec * decoder)
 {
   /* Set the output color format based on the input color format */
+  GST_OBJECT_LOCK (decoder);
   switch (decoder->param.mfx.JPEGChromaFormat) {
     case MFX_CHROMAFORMAT_YUV422:
       decoder->param.mfx.FrameInfo.FourCC = MFX_FOURCC_YUY2;
@@ -116,6 +122,7 @@ gst_msdkmjpegdec_post_configure (GstMsdkDec * decoder)
       break;
   }
 
+  GST_OBJECT_UNLOCK (decoder);
   return TRUE;
 }
 
@@ -175,11 +182,14 @@ gst_msdkdec_mjpeg_get_property (GObject * object, guint prop_id, GValue * value,
 }
 
 static void
-gst_msdkmjpegdec_class_init (GstMsdkMJPEGDecClass * klass)
+gst_msdkmjpegdec_class_init (gpointer klass, gpointer data)
 {
   GObjectClass *gobject_class;
   GstElementClass *element_class;
   GstMsdkDecClass *decoder_class;
+  MsdkDecCData *cdata = data;
+
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class = G_OBJECT_CLASS (klass);
   element_class = GST_ELEMENT_CLASS (klass);
@@ -202,11 +212,58 @@ gst_msdkmjpegdec_class_init (GstMsdkMJPEGDecClass * klass)
   gst_msdkdec_prop_install_error_report_property (gobject_class);
 #endif
 
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_msdkcaps_pad_template_init (element_class,
+      cdata->sink_caps, cdata->src_caps, doc_sink_caps_str, doc_src_caps_str);
+
+  gst_caps_unref (cdata->sink_caps);
+  gst_caps_unref (cdata->src_caps);
+  g_free (cdata);
 }
 
 static void
-gst_msdkmjpegdec_init (GstMsdkMJPEGDec * thiz)
+gst_msdkmjpegdec_init (GTypeInstance * instance, gpointer g_class)
 {
 }
+
+gboolean
+gst_msdkmjpegdec_register (GstPlugin * plugin,
+    GstMsdkContext * context, GstCaps * sink_caps,
+    GstCaps * src_caps, guint rank)
+{
+  GType type;
+  MsdkDecCData *cdata;
+  gchar *type_name, *feature_name;
+  gboolean ret = FALSE;
+
+  GTypeInfo type_info = {
+    .class_size = sizeof (GstMsdkMJPEGDecClass),
+    .class_init = gst_msdkmjpegdec_class_init,
+    .instance_size = sizeof (GstMsdkMJPEGDec),
+    .instance_init = gst_msdkmjpegdec_init
+  };
+
+  cdata = g_new (MsdkDecCData, 1);
+  cdata->sink_caps = gst_caps_copy (sink_caps);
+  cdata->src_caps = gst_caps_ref (src_caps);
+
+  gst_caps_set_simple (cdata->sink_caps, "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
+
+  GST_MINI_OBJECT_FLAG_SET (cdata->sink_caps,
+      GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+  GST_MINI_OBJECT_FLAG_SET (cdata->src_caps,
+      GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+
+  type_info.class_data = cdata;
+
+  type_name = g_strdup ("GstMsdkMJPEGDec");
+  feature_name = g_strdup ("msdkmjpegdec");
+
+  type = g_type_register_static (GST_TYPE_MSDKDEC, type_name, &type_info, 0);
+  if (type)
+    ret = gst_element_register (plugin, feature_name, rank, type);
+
+  g_free (type_name);
+  g_free (feature_name);
+
+  return ret;
+}
