@@ -31,34 +31,53 @@
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+/**
+ * SECTION: element-msdkvp9dec
+ * @title: msdkvp9dec
+ * @short_description: Intel MSDK VP9 decoderr
+ *
+ * VP9 video decoder based on Intel MFX
+ *
+ * ## Example launch line
+ * ```
+ * gst-launch-1.0 filesrc location=sample.webm ! matroskademux ! msdkvp9dec ! glimagesink
+ * ```
+ *
+ * Since: 1.16
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 #include "gstmsdkvp9dec.h"
-#include "gstmsdkvideomemory.h"
-
-#include <mfxvp9.h>
 
 GST_DEBUG_CATEGORY_EXTERN (gst_msdkvp9dec_debug);
 #define GST_CAT_DEFAULT gst_msdkvp9dec_debug
 
-#define COMMON_FORMAT "{ NV12, P010_10LE, VUYA, Y410, P012_LE, Y412_LE }"
+#define GST_MSDKVP9DEC(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_FROM_INSTANCE (obj), GstMsdkVP9Dec))
+#define GST_MSDKVP9DEC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_FROM_CLASS (klass),  GstMsdkVP9DecClass))
+#define GST_IS_MSDKVP9DEC(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_FROM_INSTANCE (obj)))
+#define GST_IS_MSDKVP9DEC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_FROM_CLASS (klass)))
 
-static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-vp9")
-    );
+/* *INDENT-OFF* */
+static const gchar *doc_src_caps_str =
+    GST_VIDEO_CAPS_MAKE (
+        "{ NV12, P010_10LE, VUYA, Y410, P012_LE, Y412_LE }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:DMABuf",
+        "{ NV12, P010_10LE, VUYA, Y410, P012_LE, Y412_LE }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:VAMemory", "{ NV12 }") " ;"
+    GST_VIDEO_CAPS_MAKE_WITH_FEATURES ("memory:D3D11Memory", "{ NV12 }");
+/* *INDENT-ON* */
 
-static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_MSDK_CAPS_STR (COMMON_FORMAT, COMMON_FORMAT))
-    );
+static const gchar *doc_sink_caps_str = "video/x-vp9";
 
-#define gst_msdkvp9dec_parent_class parent_class
-G_DEFINE_TYPE (GstMsdkVP9Dec, gst_msdkvp9dec, GST_TYPE_MSDKDEC);
+static GstElementClass *parent_class = NULL;
 
 static gboolean
 gst_msdkvp9dec_configure (GstMsdkDec * decoder)
@@ -151,11 +170,14 @@ gst_msdkvp9dec_preinit_decoder (GstMsdkDec * decoder)
 }
 
 static void
-gst_msdkvp9dec_class_init (GstMsdkVP9DecClass * klass)
+gst_msdkvp9dec_class_init (gpointer klass, gpointer data)
 {
   GObjectClass *gobject_class;
   GstElementClass *element_class;
   GstMsdkDecClass *decoder_class;
+  MsdkDecCData *cdata = data;
+
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class = G_OBJECT_CLASS (klass);
   element_class = GST_ELEMENT_CLASS (klass);
@@ -171,17 +193,63 @@ gst_msdkvp9dec_class_init (GstMsdkVP9DecClass * klass)
   gst_element_class_set_static_metadata (element_class,
       "Intel MSDK VP9 decoder",
       "Codec/Decoder/Video/Hardware",
-      "VP9 video decoder based on Intel Media SDK",
+      "VP9 video decoder based on " MFX_API_SDK,
       "Sreerenj Balachandran <sreerenj.balachandran@intel.com>");
 
   gst_msdkdec_prop_install_output_oder_property (gobject_class);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_msdkcaps_pad_template_init (element_class,
+      cdata->sink_caps, cdata->src_caps, doc_sink_caps_str, doc_src_caps_str);
+
+  gst_caps_unref (cdata->sink_caps);
+  gst_caps_unref (cdata->src_caps);
+  g_free (cdata);;
 }
 
 static void
-gst_msdkvp9dec_init (GstMsdkVP9Dec * thiz)
+gst_msdkvp9dec_init (GTypeInstance * instance, gpointer g_class)
 {
+  GstMsdkVP9Dec *thiz = GST_MSDKVP9DEC (instance);
   thiz->output_order = PROP_OUTPUT_ORDER_DEFAULT;
 }
+
+gboolean
+gst_msdkvp9dec_register (GstPlugin * plugin,
+    GstMsdkContext * context, GstCaps * sink_caps,
+    GstCaps * src_caps, guint rank)
+{
+  GType type;
+  MsdkDecCData *cdata;
+  gchar *type_name, *feature_name;
+  gboolean ret = FALSE;
+
+  GTypeInfo type_info = {
+    .class_size = sizeof (GstMsdkVP9DecClass),
+    .class_init = gst_msdkvp9dec_class_init,
+    .instance_size = sizeof (GstMsdkVP9Dec),
+    .instance_init = gst_msdkvp9dec_init
+  };
+
+  cdata = g_new (MsdkDecCData, 1);
+  cdata->sink_caps = gst_caps_ref (sink_caps);
+  cdata->src_caps = gst_caps_ref (src_caps);
+
+  GST_MINI_OBJECT_FLAG_SET (cdata->sink_caps,
+      GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+  GST_MINI_OBJECT_FLAG_SET (cdata->src_caps,
+      GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+
+  type_info.class_data = cdata;
+
+  type_name = g_strdup ("GstMsdkVP9Dec");
+  feature_name = g_strdup ("msdkvp9dec");
+
+  type = g_type_register_static (GST_TYPE_MSDKDEC, type_name, &type_info, 0);
+  if (type)
+    ret = gst_element_register (plugin, feature_name, rank, type);
+
+  g_free (type_name);
+  g_free (feature_name);
+
+  return ret;
+}
