@@ -25,6 +25,9 @@
 #include <windows.h>
 #include <initguid.h>
 #include <audioclient.h>
+#include <endpointvolume.h>
+#include <string.h>
+#include <mmdeviceapi.h>
 
 G_BEGIN_DECLS
 
@@ -48,8 +51,49 @@ gboolean  _gst_wasapi2_result (HRESULT hr,
                                const gchar * function,
                                gint line);
 
+#ifndef GST_DISABLE_GST_DEBUG
 #define gst_wasapi2_result(result) \
     _gst_wasapi2_result (result, GST_CAT_DEFAULT, __FILE__, GST_FUNCTION, __LINE__)
+#else
+#define gst_wasapi2_result(result) \
+    _gst_wasapi2_result (result, NULL, __FILE__, GST_FUNCTION, __LINE__)
+#endif
+
+typedef enum
+{
+  GST_WASAPI2_ENDPOINT_CLASS_CAPTURE = 0,
+  GST_WASAPI2_ENDPOINT_CLASS_RENDER,
+  GST_WASAPI2_ENDPOINT_CLASS_LOOPBACK_CAPTURE,
+  GST_WASAPI2_ENDPOINT_CLASS_INCLUDE_PROCESS_LOOPBACK_CAPTURE,
+  GST_WASAPI2_ENDPOINT_CLASS_EXCLUDE_PROCESS_LOOPBACK_CAPTURE,
+} GstWasapi2EndpointClass;
+
+static inline gboolean
+gst_wasapi2_is_loopback_class (GstWasapi2EndpointClass device_class)
+{
+  switch (device_class) {
+    case GST_WASAPI2_ENDPOINT_CLASS_LOOPBACK_CAPTURE:
+      return TRUE;
+    default:
+      break;
+  }
+
+  return FALSE;
+}
+
+static inline gboolean
+gst_wasapi2_is_process_loopback_class (GstWasapi2EndpointClass device_class)
+{
+  switch (device_class) {
+    case GST_WASAPI2_ENDPOINT_CLASS_INCLUDE_PROCESS_LOOPBACK_CAPTURE:
+    case GST_WASAPI2_ENDPOINT_CLASS_EXCLUDE_PROCESS_LOOPBACK_CAPTURE:
+      return TRUE;
+    default:
+      break;
+  }
+
+  return FALSE;
+}
 
 guint64       gst_wasapi2_util_waveformatex_to_channel_mask (WAVEFORMATEX * format,
                                                              GstAudioChannelPosition ** out_position);
@@ -69,6 +113,20 @@ gboolean      gst_wasapi2_can_process_loopback (void);
 
 WAVEFORMATEX * gst_wasapi2_get_default_mix_format (void);
 
+const wchar_t * gst_wasapi2_get_default_device_id_wide (EDataFlow flow);
+
+const char * gst_wasapi2_get_default_device_id (EDataFlow flow);
+
 G_END_DECLS
 
+#ifdef __cplusplus
+#include <mutex>
+
+#define GST_WASAPI2_CALL_ONCE_BEGIN \
+    static std::once_flag __once_flag; \
+    std::call_once (__once_flag, [&]()
+
+#define GST_WASAPI2_CALL_ONCE_END )
+#endif
+
 #endif /* __GST_WASAPI_UTIL_H__ */
